@@ -4,7 +4,7 @@ import os
 try:
   import time
   import sys
-  import getopt
+  from flags import Flags
   import subprocess
 
   import sys
@@ -19,6 +19,7 @@ try:
 except ImportError:
   print("Installing dependencies of build...")
   os.system("pip install tqdm")
+  os.system("pip install bashflags.py")
   import time
   import sys
   import getopt
@@ -39,20 +40,58 @@ import sys
 
 pos = ["name","run","version","dependencies","modules","projects","authors"]
 class mainz:
-  def openproj():
-    with open(f"{os.getcwd()}/build.proj") as w:
-      content = w.read()
-      if content == "":
-        with open(f"{os.getcwd()}/build.proj","w") as x:
-          x.write("{}")
-
-    with open(f"{os.getcwd()}/build.proj") as w:
-      content = w.read()
-
-      
+  def test(arg):
+    
+    if arg == "init":
+      tw = {}
+      os.system(f"mkdir {os.getcwd()}/tests")
+      os.system(f"cd {os.getcwd()}/tests && touch tests.json")
+      tw["run"] = "echo what to run for each test eg. `python` so python <testfile>"
+      tw["ignore"] = ["files to ignore eg. `.env`"]
+      with open(f"{os.getcwd()}/tests/tests.json","w") as x:
+        json.dump(tw,x)
+      printc(CGREEN,"Initalized tests!")
+    else:
         
-      x = json.loads(content)
-    return x
+      files = []
+      d = {}
+      for file in os.listdir(f"{os.getcwd()}/tests/"):
+        if file.endswith(".json") == False:
+          files.append(file)
+      with open(f"{os.getcwd()}/tests/tests.json") as x:
+        c = json.load(x)
+      tr = c["run"]
+      for item in c["ignore"]:
+        files.remove(item)
+      amt = 1
+      for item in files:
+        print(f"{amt}. {item}")
+        d[amt] = item
+        amt += 1
+      q = input("> ")
+      q = int(q)
+      printc(CRED,f"running test no.{q}")
+      printc(CWHITE,"............")
+      fr = d[q]
+      os.system(f"cd {os.getcwd()}/tests && {tr} {fr}")
+      
+  def openproj():
+    try:
+      with open(f"{os.getcwd()}/build.proj") as w:
+        content = w.read()
+        if content == "":
+          with open(f"{os.getcwd()}/build.proj","w") as x:
+            x.write("{}")
+  
+      with open(f"{os.getcwd()}/build.proj") as w:
+        content = w.read()
+  
+        
+          
+        x = json.loads(content)
+      return x
+    except:
+      return None
 
   def writeproj(proj):
     x = {}
@@ -67,7 +106,7 @@ class mainz:
 
   def structproj(): # just a ref
     
-    proj = main.openproj()
+    proj = mainz.openproj()
 
     # main struct
 
@@ -75,6 +114,7 @@ class mainz:
     author = proj["authors"]
     ver = proj["version"]
     run = proj["run"]
+    run = proj["language"]
     deps = proj["dependencies"]
     mods = proj["modules"]
 
@@ -89,7 +129,7 @@ class mainz:
 
   def parseproj(opt,arg,args): # just a ref
     
-    proj = main.openproj()
+    proj = mainz.openproj()
 
     # main struct
     try:
@@ -103,27 +143,27 @@ class mainz:
       projects = []
       for item in proj["projects"]:
         projects.append(item)
-    except KeyError:
+    except:
       if arg != "init":
         return print("ERROR: values not found")
 
 
 
-    if opt == "--proj":
+    if opt == "proj":
       if arg == "info":
         print(f"NAME: {name}\nAUTHORS: {authors}\nVERSION: {ver}")
       if arg == "init":
         os.system(f"touch {os.getcwd()}/build.proj")
-        proj = main.openproj()
+        proj = mainz.openproj()
         proj["name"] = "Name"
         proj["authors"] = []
         proj["version"] = "0.0.1"
-        proj["run"] = "echo You can use Installfile by replacing this string with 'Installfile' "
-        proj["dependencies"] = ["echo You can use Buildfile by replacing this string with 'Buildfile' ","echo eg. pip install <package>"]
+        proj["language"] = "<programming language eg. python, use full forms like javascript NOT js>"
+        proj["run"] = "echo You can use Buildfile by replacing this string with 'Buildfile' "
+        proj["dependencies"] = ["echo You can use Installfile by replacing this string with 'Installfile'","echo eg. pip install <package>"]
         proj["modules"] = ["https://pathtozipwithlocalmodules.com/module.zip"]
         proj["projects"] = []
-        print(proj)
-        main.writeproj(proj)
+        mainz.writeproj(proj)
       if arg == "set" or arg == "add":
         msg = ""
         val = args[0]
@@ -142,7 +182,7 @@ class mainz:
 
         if val not in posx:
           return print("Value does not exist in build.proj")
-        proj = main.openproj()
+        proj = mainz.openproj()
         if val in ["dependencies","modules","authors","projects"]:
           if toset in proj[val]:
             proj[val].remove(toset)
@@ -154,17 +194,17 @@ class mainz:
         else:
           proj[val] = toset
           msg = "set"
-        main.writeproj(proj)
+        mainz.writeproj(proj)
         print(f"Successfully {msg} '{val}' as '{toset}'")
       if arg == "run":
-        proj = main.openproj()
+        proj = mainz.openproj()
         run = proj["run"]
         if run == "Installfile":
           default({"x":"t","file":"Installfile","args":args})
         else:
           os.system(run)
       if arg == "dependencies" or arg == "deps":
-        proj = main.openproj()
+        proj = mainz.openproj()
         run = proj["dependencies"]
         if deps[0] == "Buildfile":
           default({"x":"t","args":args})
@@ -260,12 +300,12 @@ def send_help():
     print('USAGE: build [options]')
     print('A build tool created by Ehnryu\n')
     print('Basic options:\n')
-    print('--help : this command\n')
-    print('--build : build a project\n')
-    print('--install : install a project\n')
-    print('--sendargs : send arguments to a project\n')
-    print('--runfunc : run a specialized function\n')
-    print('--proj : configure a project\n')
+    print('help : this command\n')
+    print('build : build a project\n')
+    print('install : install a project\n')
+    print('sendargs : send arguments to a project\n')
+    print('runfunc : run a specialized function\n')
+    print('proj : configure a project\n')
 
 def printc(color,*text):
   CEND      = '\33[0m'
@@ -276,42 +316,52 @@ def printc(color,*text):
 def run(command):
     subprocess.check_output(command,shell=True)
 
-def main():
+def flags():
 
     advanced = ["help","sendargs =","runfunc =","build","install","proj ="]
 
 
-    argv = sys.argv[1:]
-    opts = []
+    argv = sys.argv
+    bashflags = ""
 
 
     try:
-      opts, args = getopt.getopt(argv, "hr:",advanced)
+      
+      bashflags = Flags(argv)
 
 
     except:
         printc(CRED,"ERROR: Invalid arguments provided.")
         send_help()
+    opt = bashflags.flag
+    arg = bashflags.arg
+    args = bashflags.args
+  
+    
 
-    for opt, arg in opts:
-        opt = opt.rstrip()
-        opt = opt.replace(" ","")
-        opt = opt.lower()
+    opt = opt.rstrip()
+    opt = opt.replace(" ","")
+    opt = opt.lower()
+    if opt in [""]:
+      return {"x":"t","args":args}
 
-        if opt in ["-h","--help"]:
-          send_help()
+    if opt in ["-h","help"]:
+      send_help()
 
-        if opt in ["--proj"]:
-          mainz.parseproj(opt,arg,args)
-          
-        if opt in ["--sendargs"]:
-          return {"x":"t","args":sys.argv}
-        if opt in ["--build"]:
-          return {"x":"t","args":args}
-        if opt in ["--install"]:
-          return {"x":"t","file":"Installfile","args":args}
-        if opt in ["--runfunc","-r"]:
-          return {"x":"t","args":args,"func":arg}
+    if opt in ["proj"]:
+      mainz.parseproj(opt,arg,args)
+
+    if opt in ["test"]:
+      mainz.test(arg)
+      
+    if opt in ["sendargs"]:
+      return {"x":"t","args":sys.argv}
+    if opt in ["build"]:
+      return {"x":"t","args":args}
+    if opt in ["install"]:
+      return {"x":"t","file":"Installfile","args":args}
+    if opt in ["runfunc","-r"]:
+      return {"x":"t","args":args,"func":arg}
 
 
 
@@ -333,7 +383,7 @@ def default(x):
       argz = ""
 
       for item in args:
-        if item not in ["build","build.py","main.py","--sendargs"]:
+        if item not in ["build","build.py","main.py","sendargs"]:
           if item != args[len(args)-1]:
             argz += f"{item},"
           else:
@@ -368,7 +418,7 @@ def default(x):
           if cs:
             iten = item
             item = item.rstrip()
-            if "--viewcomments" in args or "-vc" in args:
+            if "viewcomments" in args or "-vc" in args:
               if item.startswith("#"):
                 printc(CBEIGE,f"COMMENT: {item}")
             if item.startswith("\endfunc"):
@@ -647,7 +697,7 @@ def default(x):
 r = True 
 
 def launch():
-  x = main()
+  x = flags()
   default(x)
 
 
